@@ -5,36 +5,40 @@
  * This file contains functions to exercise shard creation functionality
  * within Citus.
  *
- * Copyright (c) 2014-2016, Citus Data, Inc.
+ * Copyright (c) Citus Data, Inc.
  *
  *-------------------------------------------------------------------------
  */
 
+#include <string.h>
+
 #include "postgres.h"
+
 #include "c.h"
 #include "fmgr.h"
 
-#include <string.h>
-
-#if (PG_VERSION_NUM >= 90500 && PG_VERSION_NUM < 90600)
 #include "access/stratnum.h"
-#else
-#include "access/skey.h"
-#endif
 #include "catalog/pg_type.h"
-#include "distributed/metadata_cache.h"
-#include "distributed/master_metadata_utility.h"
-#include "distributed/multi_join_order.h"
-#include "distributed/multi_physical_planner.h"
-#include "distributed/resource_lock.h"
-#include "distributed/test_helper_functions.h" /* IWYU pragma: keep */
+#include "nodes/makefuncs.h"
+#include "nodes/nodeFuncs.h"
+#include "nodes/nodes.h"
 #include "nodes/pg_list.h"
 #include "nodes/primnodes.h"
-#include "nodes/nodes.h"
 #include "optimizer/clauses.h"
 #include "utils/array.h"
 #include "utils/palloc.h"
 
+#include "pg_version_constants.h"
+
+#include "distributed/listutils.h"
+#include "distributed/metadata_cache.h"
+#include "distributed/metadata_utility.h"
+#include "distributed/multi_join_order.h"
+#include "distributed/multi_physical_planner.h"
+#include "distributed/resource_lock.h"
+#include "distributed/shard_pruning.h"
+#include "distributed/utils/array_type.h"
+
 
 /* local function forward declarations */
 static Expr * MakeTextPartitionExpression(Oid distributedTableId, text *value);
@@ -136,6 +140,10 @@ debug_equality_expression(PG_FUNCTION_ARGS)
 {
 	Oid distributedTableId = PG_GETARG_OID(0);
 	uint32 rangeTableId = 1;
+	if (!IsCitusTableType(distributedTableId, HASH_DISTRIBUTED))
+	{
+		ereport(ERROR, (errmsg("table needs to be hash distributed")));
+	}
 	Var *partitionColumn = PartitionColumn(distributedTableId, rangeTableId);
 	OpExpr *equalityExpression = MakeOpExpression(partitionColumn, BTEqualStrategyNumber);
 
@@ -172,7 +180,11 @@ MakeTextPartitionExpression(Oid distributedTableId, text *value)
 	if (value != NULL)
 	{
 		OpExpr *equalityExpr = MakeOpExpression(partitionColumn, BTEqualStrategyNumber);
-		Const *rightConst = (Const *) get_rightop((Expr *) equalityExpr);
+		Node *rightOp = get_rightop((Expr *) equalityExpr);
+
+		Assert(rightOp != NULL);
+		Assert(IsA(rightOp, Const));
+		Const *rightConst = (Const *) rightOp;
 
 		rightConst->constvalue = (Datum) value;
 		rightConst->constisnull = false;
@@ -201,32 +213,27 @@ MakeTextPartitionExpression(Oid distributedTableId, text *value)
 static ArrayType *
 PrunedShardIdsForTable(Oid distributedTableId, List *whereClauseList)
 {
-	ArrayType *shardIdArrayType = NULL;
-	ListCell *shardCell = NULL;
 	int shardIdIndex = 0;
 	Oid shardIdTypeId = INT8OID;
 	Index tableId = 1;
 
-	List *shardList = LoadShardIntervalList(distributedTableId);
-	int shardIdCount = -1;
-	Datum *shardIdDatumArray = NULL;
 
-	shardList = PruneShardList(distributedTableId, tableId, whereClauseList, shardList);
+	List *shardList = PruneShards(distributedTableId, tableId, whereClauseList, NULL);
 
-	shardIdCount = list_length(shardList);
-	shardIdDatumArray = palloc0(shardIdCount * sizeof(Datum));
+	int shardIdCount = list_length(shardList);
+	Datum *shardIdDatumArray = palloc0(shardIdCount * sizeof(Datum));
 
-	foreach(shardCell, shardList)
+	ShardInterval *shardInterval = NULL;
+	foreach_ptr(shardInterval, shardList)
 	{
-		ShardInterval *shardId = (ShardInterval *) lfirst(shardCell);
-		Datum shardIdDatum = Int64GetDatum(shardId->shardId);
+		Datum shardIdDatum = Int64GetDatum(shardInterval->shardId);
 
 		shardIdDatumArray[shardIdIndex] = shardIdDatum;
 		shardIdIndex++;
 	}
 
-	shardIdArrayType = DatumArrayToArrayType(shardIdDatumArray, shardIdCount,
-											 shardIdTypeId);
+	ArrayType *shardIdArrayType = DatumArrayToArrayType(shardIdDatumArray, shardIdCount,
+														shardIdTypeId);
 
 	return shardIdArrayType;
 }
@@ -239,16 +246,14 @@ PrunedShardIdsForTable(Oid distributedTableId, List *whereClauseList)
 static ArrayType *
 SortedShardIntervalArray(Oid distributedTableId)
 {
-	ArrayType *shardIdArrayType = NULL;
-	int shardIndex = 0;
 	Oid shardIdTypeId = INT8OID;
 
-	DistTableCacheEntry *cacheEntry = DistributedTableCacheEntry(distributedTableId);
+	CitusTableCacheEntry *cacheEntry = GetCitusTableCacheEntry(distributedTableId);
 	ShardInterval **shardIntervalArray = cacheEntry->sortedShardIntervalArray;
 	int shardIdCount = cacheEntry->shardIntervalArrayLength;
 	Datum *shardIdDatumArray = palloc0(shardIdCount * sizeof(Datum));
 
-	for (shardIndex = 0; shardIndex < shardIdCount; ++shardIndex)
+	for (int shardIndex = 0; shardIndex < shardIdCount; ++shardIndex)
 	{
 		ShardInterval *shardId = shardIntervalArray[shardIndex];
 		Datum shardIdDatum = Int64GetDatum(shardId->shardId);
@@ -256,8 +261,8 @@ SortedShardIntervalArray(Oid distributedTableId)
 		shardIdDatumArray[shardIndex] = shardIdDatum;
 	}
 
-	shardIdArrayType = DatumArrayToArrayType(shardIdDatumArray, shardIdCount,
-											 shardIdTypeId);
+	ArrayType *shardIdArrayType = DatumArrayToArrayType(shardIdDatumArray, shardIdCount,
+														shardIdTypeId);
 
 	return shardIdArrayType;
 }
