@@ -3,7 +3,7 @@
  * citus_nodefuncs.c
  *	  Helper functions for dealing with nodes
  *
- * Copyright (c) 2012-2016, Citus Data, Inc.
+ * Copyright (c) Citus Data, Inc.
  *
  *-------------------------------------------------------------------------
  */
@@ -11,9 +11,50 @@
 #include "postgres.h"
 
 #include "catalog/pg_type.h"
-#include "distributed/citus_nodefuncs.h"
-#include "distributed/metadata_cache.h"
 
+#include "pg_version_constants.h"
+
+#include "distributed/citus_nodefuncs.h"
+#include "distributed/citus_nodes.h"
+#include "distributed/coordinator_protocol.h"
+#include "distributed/distributed_planner.h"
+#include "distributed/errormessage.h"
+#include "distributed/log_utils.h"
+#include "distributed/metadata_cache.h"
+#include "distributed/multi_router_planner.h"
+#include "distributed/multi_server_executor.h"
+
+static const char *CitusNodeTagNamesD[] = {
+	"MultiNode",
+	"MultiTreeRoot",
+	"MultiProject",
+	"MultiCollect",
+	"MultiSelect",
+	"MultiTable",
+	"MultiJoin",
+	"MultiPartition",
+	"MultiCartesianProduct",
+	"MultiExtendedOp",
+	"Job",
+	"MapMergeJob",
+	"DistributedPlan",
+	"DistributedSubPlan",
+	"UsedDistributedSubPlan",
+	"Task",
+	"LocalPlannedStatement",
+	"ShardInterval",
+	"ShardPlacement",
+	"RelationShard",
+	"RelationRowLock",
+	"DeferredErrorMessage",
+	"GroupShardPlacement",
+	"TableDDLCommand"
+};
+
+const char **CitusNodeTagNames = CitusNodeTagNamesD;
+
+/* support for CitusNewNode() macro */
+CitusNode *newCitusNodeMacroHolder;
 
 /* exports for SQL callable functions */
 PG_FUNCTION_INFO_V1(citus_extradata_container);
@@ -35,21 +76,15 @@ PG_FUNCTION_INFO_V1(citus_extradata_container);
  * will not be handled by out/readfuncs.c. For the current uses that's ok.
  */
 void
-SetRangeTblExtraData(RangeTblEntry *rte, CitusRTEKind rteKind,
-					 char *fragmentSchemaName, char *fragmentTableName,
-					 List *tableIdList)
+SetRangeTblExtraData(RangeTblEntry *rte, CitusRTEKind rteKind, char *fragmentSchemaName,
+					 char *fragmentTableName, List *tableIdList, List *funcColumnNames,
+					 List *funcColumnTypes, List *funcColumnTypeMods,
+					 List *funcCollations)
 {
-	RangeTblFunction *fauxFunction = NULL;
-	FuncExpr *fauxFuncExpr = NULL;
-	Const *rteKindData = NULL;
-	Const *fragmentSchemaData = NULL;
-	Const *fragmentTableData = NULL;
-	Const *tableIdListData = NULL;
-
 	Assert(rte->eref);
 
 	/* store RTE kind as a plain int4 */
-	rteKindData = makeNode(Const);
+	Const *rteKindData = makeNode(Const);
 	rteKindData->consttype = INT4OID;
 	rteKindData->constlen = 4;
 	rteKindData->constvalue = Int32GetDatum(rteKind);
@@ -58,7 +93,7 @@ SetRangeTblExtraData(RangeTblEntry *rte, CitusRTEKind rteKind,
 	rteKindData->location = -1;
 
 	/* store the fragment schema as a cstring */
-	fragmentSchemaData = makeNode(Const);
+	Const *fragmentSchemaData = makeNode(Const);
 	fragmentSchemaData->consttype = CSTRINGOID;
 	fragmentSchemaData->constlen = -2;
 	fragmentSchemaData->constvalue = CStringGetDatum(fragmentSchemaName);
@@ -67,7 +102,7 @@ SetRangeTblExtraData(RangeTblEntry *rte, CitusRTEKind rteKind,
 	fragmentSchemaData->location = -1;
 
 	/* store the fragment name as a cstring */
-	fragmentTableData = makeNode(Const);
+	Const *fragmentTableData = makeNode(Const);
 	fragmentTableData->consttype = CSTRINGOID;
 	fragmentTableData->constlen = -2;
 	fragmentTableData->constvalue = CStringGetDatum(fragmentTableName);
@@ -76,7 +111,7 @@ SetRangeTblExtraData(RangeTblEntry *rte, CitusRTEKind rteKind,
 	fragmentTableData->location = -1;
 
 	/* store the table id list as an array of integers: FIXME */
-	tableIdListData = makeNode(Const);
+	Const *tableIdListData = makeNode(Const);
 	tableIdListData->consttype = CSTRINGOID;
 	tableIdListData->constbyval = false;
 	tableIdListData->constlen = -2;
@@ -95,18 +130,33 @@ SetRangeTblExtraData(RangeTblEntry *rte, CitusRTEKind rteKind,
 	}
 
 	/* create function expression to store our faux arguments in */
-	fauxFuncExpr = makeNode(FuncExpr);
+	FuncExpr *fauxFuncExpr = makeNode(FuncExpr);
 	fauxFuncExpr->funcid = CitusExtraDataContainerFuncId();
+	fauxFuncExpr->funcresulttype = RECORDOID;
 	fauxFuncExpr->funcretset = true;
 	fauxFuncExpr->location = -1;
 	fauxFuncExpr->args = list_make4(rteKindData, fragmentSchemaData,
 									fragmentTableData, tableIdListData);
 
-	fauxFunction = makeNode(RangeTblFunction);
+	RangeTblFunction *fauxFunction = makeNode(RangeTblFunction);
 	fauxFunction->funcexpr = (Node *) fauxFuncExpr;
 
 	/* set the column count to pass ruleutils checks, not used elsewhere */
-	fauxFunction->funccolcount = list_length(rte->eref->colnames);
+	if (rte->relid != 0)
+	{
+		Relation rel = RelationIdGetRelation(rte->relid);
+		fauxFunction->funccolcount = RelationGetNumberOfAttributes(rel);
+		RelationClose(rel);
+	}
+	else
+	{
+		fauxFunction->funccolcount = list_length(rte->eref->colnames);
+	}
+
+	fauxFunction->funccolnames = funcColumnNames;
+	fauxFunction->funccoltypes = funcColumnTypes;
+	fauxFunction->funccoltypmods = funcColumnTypeMods;
+	fauxFunction->funccolcollations = funcCollations;
 
 	rte->rtekind = RTE_FUNCTION;
 	rte->functions = list_make1(fauxFunction);
@@ -124,10 +174,6 @@ ExtractRangeTblExtraData(RangeTblEntry *rte, CitusRTEKind *rteKind,
 						 char **fragmentSchemaName, char **fragmentTableName,
 						 List **tableIdList)
 {
-	RangeTblFunction *fauxFunction = NULL;
-	FuncExpr *fauxFuncExpr = NULL;
-	Const *tmpConst = NULL;
-
 	/* set base rte kind first, so this can be used for 'non-extended' RTEs as well */
 	if (rteKind != NULL)
 	{
@@ -164,13 +210,13 @@ ExtractRangeTblExtraData(RangeTblEntry *rte, CitusRTEKind *rteKind,
 	}
 
 	/* should pretty much always be a FuncExpr, but be liberal in what we expect... */
-	fauxFunction = linitial(rte->functions);
+	RangeTblFunction *fauxFunction = linitial(rte->functions);
 	if (!IsA(fauxFunction->funcexpr, FuncExpr))
 	{
 		return;
 	}
 
-	fauxFuncExpr = (FuncExpr *) fauxFunction->funcexpr;
+	FuncExpr *fauxFuncExpr = (FuncExpr *) fauxFunction->funcexpr;
 
 	/*
 	 * There will never be a range table entry with this function id, but for
@@ -194,7 +240,7 @@ ExtractRangeTblExtraData(RangeTblEntry *rte, CitusRTEKind *rteKind,
 	}
 
 	/* extract rteKind */
-	tmpConst = (Const *) linitial(fauxFuncExpr->args);
+	Const *tmpConst = (Const *) linitial(fauxFuncExpr->args);
 	Assert(IsA(tmpConst, Const));
 	Assert(tmpConst->consttype == INT4OID);
 	if (rteKind != NULL)
@@ -255,7 +301,7 @@ ModifyRangeTblExtraData(RangeTblEntry *rte, CitusRTEKind rteKind,
 
 	SetRangeTblExtraData(rte, rteKind,
 						 fragmentSchemaName, fragmentTableName,
-						 tableIdList);
+						 tableIdList, NIL, NIL, NIL, NIL);
 }
 
 
@@ -268,11 +314,14 @@ GetRangeTblKind(RangeTblEntry *rte)
 	switch (rte->rtekind)
 	{
 		/* directly rtekind if it's not possibly an extended RTE */
+		case RTE_TABLEFUNC:
+		case RTE_NAMEDTUPLESTORE:
 		case RTE_RELATION:
 		case RTE_SUBQUERY:
 		case RTE_JOIN:
 		case RTE_VALUES:
 		case RTE_CTE:
+		case RTE_RESULT:
 		{
 			rteKind = (CitusRTEKind) rte->rtekind;
 			break;
@@ -304,6 +353,87 @@ Datum
 citus_extradata_container(PG_FUNCTION_ARGS)
 {
 	ereport(ERROR, (errmsg("not supposed to get here, did you cheat?")));
-
 	PG_RETURN_NULL();
 }
+
+
+static void
+CopyUnsupportedCitusNode(struct ExtensibleNode *newnode,
+						 const struct ExtensibleNode *oldnode)
+{
+	ereport(ERROR, (errmsg("not implemented")));
+}
+
+
+static bool
+EqualUnsupportedCitusNode(const struct ExtensibleNode *a,
+						  const struct ExtensibleNode *b)
+{
+	ereport(ERROR, (errmsg("not implemented")));
+}
+
+
+/* *INDENT-OFF* */
+#define DEFINE_NODE_METHODS(type) \
+	{ \
+		#type, \
+		sizeof(type), \
+		CopyNode##type, \
+		EqualUnsupportedCitusNode, \
+		Out##type, \
+		ReadUnsupportedCitusNode \
+	}
+
+#define DEFINE_NODE_METHODS_NO_READ(type) \
+	{ \
+		#type, \
+		sizeof(type), \
+		CopyUnsupportedCitusNode, \
+		EqualUnsupportedCitusNode, \
+		Out##type, \
+		ReadUnsupportedCitusNode \
+	}
+
+
+/* *INDENT-ON* */
+const ExtensibleNodeMethods nodeMethods[] =
+{
+	DEFINE_NODE_METHODS(DistributedPlan),
+	DEFINE_NODE_METHODS(DistributedSubPlan),
+	DEFINE_NODE_METHODS(UsedDistributedSubPlan),
+	DEFINE_NODE_METHODS(Job),
+	DEFINE_NODE_METHODS(ShardInterval),
+	DEFINE_NODE_METHODS(MapMergeJob),
+	DEFINE_NODE_METHODS(ShardPlacement),
+	DEFINE_NODE_METHODS(RelationShard),
+	DEFINE_NODE_METHODS(RelationRowLock),
+	DEFINE_NODE_METHODS(Task),
+	DEFINE_NODE_METHODS(LocalPlannedStatement),
+	DEFINE_NODE_METHODS(DeferredErrorMessage),
+	DEFINE_NODE_METHODS(GroupShardPlacement),
+
+	/* nodes with only output support */
+	DEFINE_NODE_METHODS_NO_READ(MultiNode),
+	DEFINE_NODE_METHODS_NO_READ(MultiTreeRoot),
+	DEFINE_NODE_METHODS_NO_READ(MultiProject),
+	DEFINE_NODE_METHODS_NO_READ(MultiCollect),
+	DEFINE_NODE_METHODS_NO_READ(MultiSelect),
+	DEFINE_NODE_METHODS_NO_READ(MultiTable),
+	DEFINE_NODE_METHODS_NO_READ(MultiJoin),
+	DEFINE_NODE_METHODS_NO_READ(MultiPartition),
+	DEFINE_NODE_METHODS_NO_READ(MultiCartesianProduct),
+	DEFINE_NODE_METHODS_NO_READ(MultiExtendedOp),
+	DEFINE_NODE_METHODS_NO_READ(TableDDLCommand)
+};
+
+void
+RegisterNodes(void)
+{
+	StaticAssertExpr(lengthof(nodeMethods) == lengthof(CitusNodeTagNamesD),
+					 "number of node methods and names do not match");
+
+	for (int off = 0; off < lengthof(nodeMethods); off++)
+	{
+		RegisterExtensibleNodeMethods(&nodeMethods[off]);
+	}
+}
