@@ -1,10 +1,12 @@
 --
 -- MULTI_SINGLE_RELATION_SUBQUERY
 --
-
 -- This test checks that we are able to run selected set of distributed SQL subqueries.
 
-SET citus.task_executor_type TO 'task-tracker';
+
+SET citus.next_shard_id TO 860000;
+
+
 
 select
     number_sum,
@@ -30,6 +32,31 @@ order by
     number_sum desc
 limit 10;
 
+-- same query above, just replace outer where clause with inner having clause
+select
+    number_sum,
+    count(*) as total,
+    avg(total_count) avg_count
+from
+    (select
+        l_suppkey,
+        l_linestatus,
+        sum(l_linenumber) as number_sum,
+        count(*) as total_count
+    from
+        lineitem
+    group by
+        l_suppkey,
+        l_linestatus
+    having
+        sum(l_linenumber) >= 10) as distributed_table
+group by
+    number_sum
+order by
+    total desc,
+    number_sum desc
+limit 10;
+
 select
     (l_suppkey / 100) as suppkey_bin,
     avg(total_count) avg_count
@@ -46,7 +73,7 @@ from
 group by
     suppkey_bin
 order by
-    avg_count desc
+    avg_count desc, suppkey_bin DESC
 limit 20;
 
 select
@@ -94,8 +121,7 @@ from
         (l_orderkey/4)::int,
         l_suppkey )  as distributed_table;
 
--- Check that we don't support subqueries with limit.
-
+-- we don't support subqueries with limit.
 select
     l_suppkey,
     sum(suppkey_count) as total_suppkey_count
@@ -111,19 +137,23 @@ from
         l_suppkey
     limit 100) as distributed_table
 group by
-    l_suppkey;
+    l_suppkey
+    ORDER BY 2 DESC, 1 DESC
+LIMIT 5;
 
 -- Check that we don't support subqueries without aggregates.
 
 select
-    rounded_tax
+    DISTINCT rounded_tax
 from
     (select
         round(l_tax) as rounded_tax
     from
         lineitem
     group by
-        l_tax) as distributed_table;
+        l_tax) as distributed_table
+    ORDER BY 1 DESC
+    LIMIT 5;
 
 -- Check that we support subqueries with count(distinct).
 
@@ -137,6 +167,18 @@ from
     group by
         l_partkey) as distributed_table;
 
+select
+    avg(different_shipment_days)
+from
+    (select
+        count(distinct l_shipdate) as different_shipment_days
+    from
+        lineitem
+    group by
+        l_partkey
+    having
+        count(distinct l_shipdate) >= 2) as distributed_table;
+
 -- Check that if subquery is pulled, we don't error and run query properly.
 
 SELECT max(l_suppkey) FROM
@@ -148,7 +190,7 @@ SELECT max(l_suppkey) FROM
                 l_suppkey,
                 count(*)
             FROM
-                lineitem_subquery
+                lineitem
             WHERE
                 l_orderkey < 20000
             GROUP BY
